/* color triangle (uniform) */
#include <cmath>
#include <iostream>
#include <GL/glew.h>
#include <GL/freeglut.h>
#include <GL/freeglut_ext.h>
#include "moglclass.h"
#include "shaders.h"
#include "vertexBufferObject.h"

using namespace moglclass;

#define NUMSHADERS 2
Shader shShaders[NUMSHADERS];
Program spMain;

GLuint vao;
VertexBufferObject vbo;

glm::vec4 vertices[] = {
	glm::vec4(-0.75f, -0.75f, 0.0f, 1.0f),
	glm::vec4(0.75f, -0.75f, 0.0f, 1.0f),
	glm::vec4(0.75f, 0.75f, 0.0f, 1.0f)
};
glm::vec4 colors[] = {
	glm::vec4(1.0f, 0.0f, 0.0f, 1.0f),
	glm::vec4(0.0f, 1.0f, 0.0f, 1.0f),
	glm::vec4(0.0f, 0.0f, 1.0f, 1.0f)
};

bool g_wiremode = false;

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

// set color triangle
void setTriangle()
{
	// VBO
	vbo.addData(&vertices[0], sizeof(glm::vec4)); // vertex1 position
	vbo.addData(&colors[0], sizeof(glm::vec4));		// vertex1 color
	vbo.addData(&vertices[1], sizeof(glm::vec4));	// vertex2 position
	vbo.addData(&colors[1], sizeof(glm::vec4));		// vertex2 color
	vbo.addData(&vertices[2], sizeof(glm::vec4));	// vertex3 position
	vbo.addData(&colors[2], sizeof(glm::vec4));		// vertex3 color

	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	// VAO
	glGenVertexArrays(1, &vao);
	printf("vao=%d\n", vao);
	glBindVertexArray(vao);

	// vertex attrib
	int iDataStride = 2 * sizeof(glm::vec4); // vertex & color
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec4);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 4, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
}

void init( void )
{
    // Load shaders and use the resulting shader program
	spMain.createProgram();
	shShaders[0].loadShader("simple1.vs", GL_VERTEX_SHADER);
	shShaders[1].loadShader("simple1.fs", GL_FRAGMENT_SHADER);
	spMain.attachShaderToProgram(&shShaders[0]);
	spMain.attachShaderToProgram(&shShaders[1]);
	spMain.linkProgram();
	spMain.useProgram();

	setTriangle();

	glClearColor( 1.0, 1.0, 1.0, 1.0 ); // white background
}

void drawTriangle(bool wireframe = false)
{
	glBindVertexArray(vao);
	if (wireframe)
		glDrawArrays(GL_LINE_LOOP, 0, 3);
	else
		glDrawArrays(GL_TRIANGLES, 0, 3);
}

void display( void )
{
	glClearColor( 1.0, 1.0, 1.0, 1.0 ); // white background
	glClear( GL_COLOR_BUFFER_BIT ); // clear the window
									
	drawTriangle(g_wiremode);	// draw triangle

	glFlush();
}

void keyboard( unsigned char key, int x, int y )
{
	switch ( key ) {
		case 27:
			exit( EXIT_SUCCESS );
			break;
		case 'w':
			g_wiremode = !g_wiremode;
			printf("g_wiremode=%d\n", g_wiremode);
			break;
	}
	glutPostRedisplay();
}

void cleanup()
{		 
	spMain.deleteProgram();
}

int main( int argc, char **argv )
{ 
	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_RGBA );
	glutInitWindowSize( 512, 512 );
	glutCreateWindow( "Your First TRIANGLE" );

	glewInit();
	init();
	glutDisplayFunc( display );
	glutKeyboardFunc( keyboard );
	glutMainLoop();

	cleanup();
	return 0;
}